extension MapExt<K, V> on Map<K, V> {
  Map<K, V> where(bool Function(K key, V value) f) =>
      Map<K, V>.fromEntries(entries.where((e) => f(e.key, e.value)));

  Map<K, V> whereKey(bool Function(K key) f) =>
      Map<K, V>.fromEntries(entries.where((e) => f(e.key)));

  Map<K, V> whereValue(bool Function(V value) f) =>
      Map<K, V>.fromEntries(entries.where((e) => f(e.value)));
}

extension IterableExt<T> on Iterable<T> {
  Iterable<E> mapWithIndex<E>(E Function(int index, T value) f) =>
      Iterable.generate(length).map((index) => f(index, elementAt(index)));
}

extension ListExt<T> on List<T> {
  Map<T, E> toMap<E>(E Function(T) f) => {for (var key in this) key: f(key)};
}

extension ListMinMax<T extends num> on List<T> {
  T get min => _sorted.first;

  T get max => _sorted.last;

  List<T> get _sorted => [...this]..sort();
}
