import 'package:firebase_auth/firebase_auth.dart';
import 'package:firebase_database/firebase_database.dart';
import 'package:flutter/foundation.dart';
import 'package:wowit/interfaces/auth/auth.dart';

mixin FirebaseLoginMixin on BaseAuth {
  Future<User?> loginFirebase() async {
    try {
      User? user = FirebaseAuth.instance.currentUser;
      if (user == null) return null;
      if (kDebugMode) {
        print(
            '$runtimeType.loginFirebase with phone: ${user.phoneNumber}, uid:  ${user.uid}');
      }
      return user;
    } catch (e) {
      handelException(e, subTitle: 'loginFirebase');
      return null;
    }
  }

  Future logoutFirebase() async {
    try {
      if (kDebugMode) {
        print('$runtimeType.logoutFirebase');
      }
      await FirebaseAuth.instance.signOut();
    } catch (e) {
      handelException(e, subTitle: 'logoutFirebase');
    }
  }

  Future initAccountFirebase({
    required String phone,
    required String uidPhone,
  }) async {
    if (phone.isEmpty || uidPhone.isEmpty) return;
    try {
      final DatabaseReference db = FirebaseDatabase.instance.ref();
      await db
          .child('tblAccount/$uidPhone')
          .set({'phone': phone, 'uidPhone': uidPhone});
      if (kDebugMode) {
        print(
            '$runtimeType.initAccountFirebase with phone: $phone, uid: $uidPhone');
      }
    } catch (e) {
      handelException(e, subTitle: 'initAccountFirebase');
    }
  }
}
