import 'dart:async';

import 'package:wowit/interfaces/mixins/handle_exception_mixin.dart';
import 'package:wowit/interfaces/setting/mixins/image_init_mixin.dart';
import 'package:wowit/interfaces/setting/mixins/message_init_mixin.dart';
import 'package:wowit/interfaces/setting/mixins/parameter_init_mixin.dart';
import 'package:wowit/interfaces/setting/mixins/picker_init_mixin.dart';

abstract class BaseSetting
    with
        HandleExceptionMixin,
        MessageInitMixin,
        ImageInitMixin,
        PickerInitMixin,
        ParameterInitMixin {
  Future initData({
    bool isStepByStep = false,
    Function()? onEachSuccess,
  }) async {
    if (isStepByStep) {
      await _loadSingle(progress: initProccess, onEachSuccess: onEachSuccess);
    } else {
      await _loadMutil(progress: initProccess, onEachSuccess: onEachSuccess);
    }
  }

  /// Override lại thứ tự/ thêm bớt,... nếu cần
  List<Future Function()> get initProccess => [
        () => initMessage(),
        () => initImage(),
        () => initPicker(),
      ];

  Future _loadSingle({
    required List<Future Function()> progress,
    Function()? onEachSuccess,
  }) async {
    await Future.forEach<Future Function()>(
      initProccess,
      (future) async {
        await future.call();
        onEachSuccess?.call();
      },
    );
  }

  Future _loadMutil({
    required List<Future Function()> progress,
    Function()? onEachSuccess,
  }) async {
    for (var function in progress) {
      await function.call().whenComplete(
        () {
          onEachSuccess?.call();
        },
      );
    }
  }
}
