import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:shared_preferences/shared_preferences.dart';
import 'package:wowit/model.dart';
import 'package:wowit/sql_service.dart';
import 'common.dart';
import 'methods.dart';
import 'package:collection/collection.dart';

class PickerRange {
  PickerRange._();
  static const String _picker = 'picker';
  static const String _range = 'range';

  static const String _splitTable = '[::]';
  static const String _splitValue = '[..]';
  static const String _splitData = '[,,]';

  static Picker getDefaultPicker({required String tableName}) {
    if (kDebugMode) {
      return Picker(-1, -1, 'Chưa nhập liệu cho drop dơn này: $tableName');
    } else {
      return Picker(-1, -1, '--');
    }
  }

  static init(
    Map<String, List<Map<String, Object?>?>?> reponse, {
    bool reload = false,
  }) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> pickers = prefs.getStringList(_picker) ?? [];
    List<String> ranges = prefs.getStringList(_range) ?? [];
    if (pickers.isEmpty || ranges.isEmpty) {
      reload = true;
    }
    if (reload) {
      try {
        if (reponse.isEmpty) {
          reponse = await SQLService.execute(
            'sp_InitSetting',
            params: {'@getPickerRange': true},
          );
          if (reponse.isEmpty) {
            return;
          }
        }
        reponse.forEach((key, value) {
          String tableName = key;
          if (kDebugMode) {
            print('Picker-Range init: $tableName');
          }
          if (tableName.startsWith('DataRange_')) {
            PickerRange.initDataRange(
              tableName.replaceAll('DataRange_', ''),
              value,
              override: true,
            );
          } else if (tableName.startsWith('DataPicker_')) {
            PickerRange.initDataPicker(
              tableName.replaceAll('DataPicker_', ''),
              value,
              override: true,
            );
          }
        });
        savePickerToLocal();
        saveRangeToLocal();
        await SQLService.execute('sp_updatePicker');
        return;
      } catch (ex) {
        if (kDebugMode) {
          print('PickerRange init exception: $ex');
        }
      }
    } else {
      loadPickerFromLocal(pickers: pickers);
      loadRangeFromLocal(ranges: ranges);
    }
  }

  static String getTitle(
    String tableName,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return getDataPicker(
      getValueDataPicker(tableName),
      id,
      defaultValue: defaultValue,
    );
  }

  static String getDataPicker(
    List<Picker> data,
    int? id, {
    String defaultValue = '',
  }) {
    if (id == null) return defaultValue;
    return data.firstWhereOrNull((e) => e.id == id)?.title ?? defaultValue;
    // for (int i = 0; i < data.length; i++) {
    //   try {
    //     if (data[i].valueMember == id) {
    //       return data[i].displayMember;
    //     }
    //   } catch (e) {
    //     if (kDebugMode) {
    //       HandleException.log(e, subTitle: 'getDataPicker');
    //       return defaultValue;
    //     }
    //   }
    // }

    // return defaultValue;
  }

  static List<Picker> getValueDataPicker(
    String tableName, {
    int? parentId,
  }) {
    if (Common.pickers.containsKey(tableName)) {
      if (parentId == null || parentId < 1) {
        return Common.pickers[tableName] ?? [];
      }
      List<Picker> data = Common.pickers[tableName] ?? [];
      List<Picker> lst = [];
      for (int i = 0; i < data.length; i++) {
        if (data[i].refID == parentId) {
          lst.add(data[i]);
        }
      }
      if (lst.isEmpty) {
        lst.add(PickerRange.getDefaultPicker(tableName: tableName));
        if (kDebugMode) {
          SQLService.execute('sp_AddPickerRange',
              params: {'@tableName': tableName});
        }
      }
      return lst;
    } else {
      if (kDebugMode) {
        SQLService.execute(
          'sp_AddPickerRange',
          params: {
            '@tableName': tableName,
          },
        );
      }
    }
    return [
      PickerRange.getDefaultPicker(tableName: tableName),
    ];
  }

  static Future<List<Picker>> initDataPicker(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.pickers.containsKey(tableName) && !override) {
      return Common.pickers[tableName] ?? [];
    }
    List<Picker> listVal = [];
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute(
          'sp_getValueDataPicker',
          params: {
            '@tableName': tableName,
          },
        );
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataPicker'] ?? ds.values.first;
      }
      dtData ??= [];

      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        if (id < 0) continue;
        listVal.add(
          Picker.data(row),
        );
      }
      Common.pickers[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] getValueDataPicker: $ex');
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: $tableName count: ${listVal.length}');
    }
    return listVal;
  }

  static Map<int, RangeValue> getValueDataRange(String tableName) {
    if (Common.mapRange.containsKey(tableName)) {
      return Common.mapRange[tableName] ?? {};
    } else if (kDebugMode) {
      SQLService.execute(
        'sp_AddPickerRange',
        params: {'@tableName': tableName},
      );
    }
    return {};
  }

  static Future<Map<int, RangeValue>> initDataRange(
    String tableName,
    List<Map<String, Object?>?>? dtData, {
    bool override = false,
  }) async {
    if (Common.mapRange.containsKey(tableName) && !override) {
      return Common.mapRange[tableName] ?? {};
    }
    Map<int, RangeValue> listVal = {};
    try {
      if (dtData == null || dtData.isEmpty) {
        var ds = await SQLService.execute(
          'sp_getValueDataRange',
          params: {'@tableName': tableName},
        );
        if (ds.isEmpty) return listVal;
        dtData = ds['sp_getValueDataRange'] ?? ds.values.first;
      }
      dtData ??= [];
      for (var row in dtData) {
        int id = Methods.getInt(row, 'ID', defaultValue: -1);
        String title = Methods.getString(row, 'title');
        double from = Methods.getDouble(row, 'valFrom', defaultValue: 0);
        double to = Methods.getDouble(row, 'valTo', defaultValue: 0);
        if (id < 0) continue;
        listVal[id] = RangeValue(from, to, title: title);
      }
      Common.mapRange[tableName] = listVal;
    } catch (ex) {
      if (kDebugMode) {
        print('[Exception] initDataRange: $ex');
      }
    }
    if (kDebugMode) {
      print('getValueDataPicker: $tableName count: ${listVal.length}');
    }
    return listVal;
  }

  static RangeValue getRangeValueDataPicker(
      Map<int, RangeValue> rangeMap, int id,
      {RangeValue defaultValue = const RangeValue(0, 0)}) {
    return rangeMap[id] ?? defaultValue;
  }

  static void savePickerToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    // save name table pickers
    List<String> pickers = Common.pickers.keys.toList();
    await prefs.setStringList(_picker, pickers);

    Common.pickers.forEach((key, value) {
      List<String> dataPickers = value.map((e) => Picker.toJson(e)).toList();
      unawaited(prefs.setStringList(key, dataPickers));
    });

    if (kDebugMode) {
      print('save Picker to local success');
    }
  }

  static void saveRangeToLocal() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    List<String> ranges = Common.mapRange.entries.map<String>((e) {
      return e.key +
          _splitTable +
          e.value.entries.map<String>((e) {
            String id = e.key.toString();
            String start = e.value.start.toString();
            String end = e.value.start.toString();
            String title = e.value.title;
            title = title.isNotEmpty ? title : ' ';
            return id +
                _splitValue +
                start +
                _splitValue +
                end +
                _splitValue +
                title;
          }).join(_splitData);
    }).toList();

    await prefs.setStringList(_range, ranges);
    if (kDebugMode) {
      print('save Range to local success');
    }
  }

  static void loadPickerFromLocal({List<String>? pickers}) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    if (pickers == null || pickers.isEmpty) {
      pickers = prefs.getStringList(_picker) ?? [];
    }
    Common.pickers = {
      for (var e in pickers)
        e: prefs
                .getStringList(e)
                ?.map((json) => Picker.fromJson(json))
                .toList() ??
            [],
    };
    if (kDebugMode) {
      print('//==============================');
      print('||load Picker from local success');
      Common.pickers.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\==============================');
    }
  }

  static void loadRangeFromLocal({List<String>? ranges}) async {
    if (ranges == null || ranges.isEmpty) {
      SharedPreferences prefs = await SharedPreferences.getInstance();
      ranges = prefs.getStringList(_range) ?? [];
    }
    Common.mapRange = Map.fromIterable(
      ranges,
      key: (e) {
        String key = e as String;
        return key.split(_splitTable).first;
      },
      value: (e) {
        String dataset = e as String;
        return Map.fromIterable(
          dataset.split(_splitTable).last.split(_splitData),
          key: (e) {
            String key = e as String;
            return Methods.toInt(key.split(_splitValue).first);
          },
          value: (e) {
            String rangeValue = e as String;
            List<String> splits = rangeValue.split(_splitValue);
            double start = Methods.toDouble(splits[1]);
            double end = Methods.toDouble(splits[2]);
            String title = splits[3];
            return RangeValue(start, end, title: title);
          },
        );
      },
    );
    if (kDebugMode) {
      print('//===========================');
      print('load Range from local success');
      Common.mapRange.forEach((key, value) {
        if (kDebugMode) {
          print('||$key: ${value.length} item');
        }
      });
      print('\\\\===========================');
    }
  }
}
